#!perl
# This file was automatically generated by Dist::Zilla::Plugin::PodCoverageTests
use strict;
use warnings;
use Test::Pod::Coverage 1.08;
use Pod::Coverage::TrustPod;

all_pod_coverage_ok({ coverage_class => 'Pod::Coverage::TrustPod' });
